Package: NPARC
Type: Package
Title: Non-parametric analysis of response curves for thermal proteome
        profiling experiments
Version: 1.22.0
Author: 
    Dorothee Childs,
    Nils Kurzawa
Maintainer: Nils Kurzawa <nilskurzawa@gmail.com>
Description: Perform non-parametric analysis of response curves as described by Childs, Bach, Franken et al. (2019): Non-parametric analysis of thermal proteome profiles reveals novel drug-binding proteins.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.1.1
Depends: R (>= 4.0.0)
Imports: dplyr, tidyr, BiocParallel, broom, MASS, rlang, magrittr,
        stats, methods
Suggests: testthat, devtools, knitr, rprojroot, rmarkdown, ggplot2,
        BiocStyle
VignetteBuilder: knitr
biocViews: Software, Proteomics
git_url: https://git.bioconductor.org/packages/NPARC
git_branch: RELEASE_3_22
git_last_commit: 92e6972
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:29:58 UTC; biocbuild
