# Generated by roxygen2: do not edit by hand

export(NPARCfit)
export(NPARCtest)
export(fitSingleSigmoid)
export(getParams)
export(runNPARC)
import(BiocParallel)
import(dplyr)
import(tidyr)
importFrom(MASS,fitdistr)
importFrom(broom,augment)
importFrom(magrittr,"%<>%")
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,D)
importFrom(stats,coefficients)
importFrom(stats,fitted)
importFrom(stats,integrate)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nls)
importFrom(stats,nls.control)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,vcov)
