#' TPP dataset of staurosporine treated cells.
#'
#' Data from a thermal proteome profiling (TPP) experiment investigating the
#' ATP-competitive pan-kinase inhibitor staurosporine on K562 cells.
#' The data has been downloaded the data from the supplement of the respective publication and converted into tidy format.
#'
#' @docType data
#'
#' @usage data(stauro_TPP_data_tidy)
#'
#' @format An object of class \code{"data.frame"}
#'
#' @keywords datasets
#'
#' @references  Savitski et al. (2014): Tracking cancer drugs in living cells by thermal profiling of the proteome. Science 346, 1255784.
"stauro_TPP_data_tidy"
